// *********
// Settings
// *********
class SettingsClass {
    constructor() {

    }
}
var Settings = new SettingsClass();

function SettingsToJSON() {
    return JSON.stringify(Settings);
}

function JSONToSettings(json) {
    Settings = JSON.parse(json);
}

// *********
// Vec2
// *********
class Vec2 {
    constructor(x, y) {
        this.x = x;
        this.y = y;
    }
    normalize() {
        var n = this.length();
        if (n == 0) {
            return new Vec2(0, 0);
        }
        n = 1.0 / n;
        return new Vec2(this.x * n, this.y * n);
    }

    length() {
        return Math.sqrt(this.x * this.x + this.y * this.y);
    }

    sqrLength() {
        return this.x * this.x + this.y * this.y;
    }

    add(v) {
        return new Vec2(this.x + v.x, this.y + v.y);
    }

    sub(v) {
        return new Vec2(this.x - v.x, this.y - v.y);
    }

    dot(v) {
        return this.x * v.x + this.y * v.y;
    }

    distance(target) {
        var v = this.sub(target);
        return v.length();
    }

    scale(factor) {
        return new Vec2(this.x * factor, this.y * factor);
    }

    lerp(target, alpha) {
        var a = this.scale(1 - alpha);
        var b = target.scale(alpha);
        return a.add(b);
    }

    static forAngle(angle) {
        return new Vec2(Math.cos(angle), Math.sin(angle));
    }

    rotate(other) {
        return new Vec2(this.x * other.x - this.y * other.y, this.x * other.y + this.y * other.x);
    }

    rotateByAngle(pivot, angle) {
        let p = this;
        p.x -= pivot.x;
        p.y -= pivot.y;

        p = p.rotate(Vec2.forAngle(angle));

        p.x += pivot.x;
        p.y += pivot.y;

        return p;
    }
    
    isXEmpty() {
        return this.x == null;
    }
    
    isYEmpty() {
        return this.y == null;
    }

    toString() {
        return (
            this.x.toFixed(3).padStart(10) +
            this.y.toFixed(3).padStart(10));
    }
    static zero() { return new Vec2(0, 0); }
    static one() { return new Vec2(1, 1); }
}
// *********
// Vec3
// *********

class Vec3 {
    constructor(x, y, z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    normalize() {
        var n = this.length();
        if (n == 0) {
            return new Vec3(0, 0, 0);
        }
        n = 1.0 / n;
        return new Vec3(this.x * n, this.y * n, this.z * n);
    }

    length() {
        return Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
    }

    sqrLength() {
        return this.x * this.x + this.y * this.y + this.z * this.z;
    }

    add(v) {
        return new Vec3(this.x + v.x, this.y + v.y, this.z + v.z);
    }

    sub(v) {
        return new Vec3(this.x - v.x, this.y - v.y, this.z - v.z);
    }
    dot(v) {
        return this.x * v.x + this.y * v.y + this.z * v.z;
    }
    cross(v) {
        return new Vec3(
            this.y * v.z - this.z * v.y,
            -this.x * v.z + this.z * v.x,
            this.x * v.y - this.y * v.x
        );
    }

    distance(target) {
        var v = this.sub(target);
        return v.length();
    }

    scale(factor) {
        return new Vec3(this.x * factor, this.y * factor, this.z * factor);
    }

    lerp(target, alpha) {
        var a = this.scale(1 - alpha);
        var b = target.scale(alpha);
        return a.add(b);
    }
    
    isXEmpty() {
        return this.x == null;
    }
    
    isYEmpty() {
        return this.y == null;
    }
    
    isZEmpty() {
        return this.z == null;
    }
    
    toString() {
        return (
            this.x.toFixed(3).padStart(8) +
            this.y.toFixed(3).padStart(8) +
            this.z.toFixed(3).padStart(8));
    }
    static zero() { return new Vec3(0, 0, 0); }
    static one() { return new Vec3(1, 1, 1); }
}
// This is needed for SpiderMonkey engine to get object from Global and then use it as constructor
var JSVec3 = Vec3;
var JSVec2 = Vec2;


// *********
// Additional Math functions
// *********
//Converts from degrees to radians.
Math.radians = function (degrees) {
    return degrees * Math.PI / 180;
};

//Converts from radians to degrees.
Math.degrees = function (radians) {
    return radians * 180 / Math.PI;
};

// *********
// Misc
// *********
if (!String.prototype.padStart) {
    String.prototype.padStart = function padStart(targetLength, padString) {
        targetLength = targetLength >> 0; //truncate if number, or convert non-number to 0;
        padString = String(typeof padString !== 'undefined' ? padString : ' ');
        if (this.length >= targetLength) {
            return String(this);
        } else {
            targetLength = targetLength - this.length;
            if (targetLength > padString.length) {
                padString += padString.repeat(targetLength / padString.length); //append to original to ensure we are longer than needed
            }
            return padString.slice(0, targetLength) + String(this);
        }
    };
}

if (!Number.prototype.clamp) {
    Number.prototype.clamp = function (min, max) {
        return Math.min(Math.max(this, min), max);
    };
}

EasingFunctions = {
    linear: function (n) { return n; },
    inQuad: function (n) { return n * n; },
    outQuad: function (n) { return n * (2 - n); },
    inOutQuad: function (n) {
        n *= 2;
        if (n < 1) return 0.5 * n * n;
        return - 0.5 * (--n * (n - 2) - 1);
    },
    inCube: function (n) { return n * n * n; },
    outCube: function (n) {
        return --n * n * n + 1;
    },
    inOutCube: function (n) {
        n *= 2;
        if (n < 1) return 0.5 * n * n * n;
        return 0.5 * ((n -= 2) * n * n + 2);
    },

    inQuart: function (n) {
        return n * n * n * n;
    },

    outQuart: function (n) {
        return 1 - (--n * n * n * n);
    },

    inOutQuart: function (n) {
        n *= 2;
        if (n < 1) return 0.5 * n * n * n * n;
        return -0.5 * ((n -= 2) * n * n * n - 2);
    },

    inQuint: function (nmissil) {
        return n * n * n * n * n;
    },

    outQuint: function (n) {
        return --n * n * n * n * n + 1;
    },

    inOutQuint: function (n) {
        n *= 2;
        if (n < 1) return 0.5 * n * n * n * n * n;
        return 0.5 * ((n -= 2) * n * n * n * n + 2);
    },

    inSine: function (n) {
        return 1 - Math.cos(n * Math.PI / 2);
    },

    outSine: function (n) {
        return Math.sin(n * Math.PI / 2);
    },

    inOutSine: function (n) {
        return .5 * (1 - Math.cos(Math.PI * n));
    },

    inExpo: function (n) {
        return 0 == n ? 0 : Math.pow(1024, n - 1);
    },

    outExpo: function (n) {
        return 1 == n ? n : 1 - Math.pow(2, -10 * n);
    },

    inOutExpo: function (n) {
        if (0 == n) return 0;
        if (1 == n) return 1;
        if ((n *= 2) < 1) return .5 * Math.pow(1024, n - 1);
        return .5 * (-Math.pow(2, -10 * (n - 1)) + 2);
    },

    inCirc: function (n) {
        return 1 - Math.sqrt(1 - n * n);
    },

    outCirc: function (n) {
        return Math.sqrt(1 - (--n * n));
    },

    inOutCirc: function (n) {
        n *= 2
        if (n < 1) return -0.5 * (Math.sqrt(1 - n * n) - 1);
        return 0.5 * (Math.sqrt(1 - (n -= 2) * n) + 1);
    },

    inBack: function (n) {
        var s = 1.70158;
        return n * n * ((s + 1) * n - s);
    },

    outBack: function (n) {
        var s = 1.70158;
        return --n * n * ((s + 1) * n + s) + 1;
    },

    inOutBack: function (n) {
        var s = 1.70158 * 1.525;
        if ((n *= 2) < 1) return 0.5 * (n * n * ((s + 1) * n - s));
        return 0.5 * ((n -= 2) * n * ((s + 1) * n + s) + 2);
    },

    inBounce: function (n) {
        return 1 - EasingFunctions.outBounce(1 - n);
    },

    outBounce: function (n) {
        if (n < (1 / 2.75)) {
            return 7.5625 * n * n;
        } else if (n < (2 / 2.75)) {
            return 7.5625 * (n -= (1.5 / 2.75)) * n + 0.75;
        } else if (n < (2.5 / 2.75)) {
            return 7.5625 * (n -= (2.25 / 2.75)) * n + 0.9375;
        } else {
            return 7.5625 * (n -= (2.625 / 2.75)) * n + 0.984375;
        }
    },

    inOutBounce: function (n) {
        if (n < .5) return EasingFunctions.inBounce(n * 2) * .5;
        return EasingFunctions.outBounce(n * 2 - 1) * .5 + .5;
    },

    inElastic: function (n) {
        var s, a = 0.1, p = 0.4;
        if (n === 0) return 0;
        if (n === 1) return 1;
        if (!a || a < 1) { a = 1; s = p / 4; }
        else s = p * Math.asin(1 / a) / (2 * Math.PI);
        return - (a * Math.pow(2, 10 * (n -= 1)) * Math.sin((n - s) * (2 * Math.PI) / p));
    },

    outElastic: function (n) {
        var s, a = 0.1, p = 0.4;
        if (n === 0) return 0;
        if (n === 1) return 1;
        if (!a || a < 1) { a = 1; s = p / 4; }
        else s = p * Math.asin(1 / a) / (2 * Math.PI);
        return (a * Math.pow(2, - 10 * n) * Math.sin((n - s) * (2 * Math.PI) / p) + 1);
    },

    inOutElastic: function (n) {
        var s, a = 0.1, p = 0.4;
        if (n === 0) return 0;
        if (n === 1) return 1;
        if (!a || a < 1) { a = 1; s = p / 4; }
        else s = p * Math.asin(1 / a) / (2 * Math.PI);
        if ((n *= 2) < 1) return - 0.5 * (a * Math.pow(2, 10 * (n -= 1)) * Math.sin((n - s) * (2 * Math.PI) / p));
        return a * Math.pow(2, -10 * (n -= 1)) * Math.sin((n - s) * (2 * Math.PI) / p) * 0.5 + 1;
    }
}

//// https://github.com/davidbau/seedrandom
!function (a, b) { var l, c = eval("this"), d = 256, g = "random", h = b.pow(d, 6), i = b.pow(2, 52), j = 2 * i, k = d - 1; function m(r, t, e) { var u = [], f = q(function n(r, t) { var e, o = [], i = typeof r; if (t && "object" == i) for (e in r) try { o.push(n(r[e], t - 1)) } catch (n) { } return o.length ? o : "string" == i ? r : r + "\0" }((t = 1 == t ? { entropy: !0 } : t || {}).entropy ? [r, s(a)] : null == r ? function () { try { var n; return l && (n = l.randomBytes) ? n = n(d) : (n = new Uint8Array(d), (c.crypto || c.msCrypto).getRandomValues(n)), s(n) } catch (n) { var r = c.navigator, t = r && r.plugins; return [+new Date, c, t, c.screen, s(a)] } }() : r, 3), u), p = new n(u), m = function () { for (var n = p.g(6), r = h, t = 0; n < i;)n = (n + t) * d, r *= d, t = p.g(1); for (; j <= n;)n /= 2, r /= 2, t >>>= 1; return (n + t) / r }; return m.int32 = function () { return 0 | p.g(4) }, m.quick = function () { return p.g(4) / 4294967296 }, m.double = m, q(s(p.S), a), (t.pass || e || function (n, r, t, e) { return e && (e.S && o(e, p), n.state = function () { return o(p, {}) }), t ? (b[g] = n, r) : n })(m, f, "global" in t ? t.global : this == b, t.state) } function n(n) { var r, t = n.length, u = this, e = 0, o = u.i = u.j = 0, i = u.S = []; for (t || (n = [t++]); e < d;)i[e] = e++; for (e = 0; e < d; e++)i[e] = i[o = k & o + n[e % t] + (r = i[e])], i[o] = r; (u.g = function (n) { for (var r, t = 0, e = u.i, o = u.j, i = u.S; n--;)r = i[e = k & e + 1], t = t * d + i[k & (i[e] = i[o = k & o + r]) + (i[o] = r)]; return u.i = e, u.j = o, t })(d) } function o(n, r) { return r.i = n.i, r.j = n.j, r.S = n.S.slice(), r } function q(n, r) { for (var t, e = n + "", o = 0; o < e.length;)r[k & o] = k & (t ^= 19 * r[k & o]) + e.charCodeAt(o++); return s(r) } function s(n) { return String.fromCharCode.apply(0, n) } if (b["seed" + g] = m, q(b.random(), a), "object" == typeof module && module.exports) { module.exports = m; try { l = require("crypto") } catch (n) { } } else "function" == typeof define && define.amd && define(function () { return m }) }([], Math);




